/*----------------------------------------------------------------------
  File    : lvqd.h
  Contents: learning vector quantization demonstration
  Author  : Christian Borgelt
  History : 2000.09.01 file created from file xlvq.c
            2000.09.02 function lvq_load added
            2001.11.08 table functions moved to module tab4vis
            2003.09.02 field 'cnt' added to REFVEC structure
            2013.08.08 adapted to definitions ATTID, VALID, WEIGHT etc.
----------------------------------------------------------------------*/
#ifndef __LVQD__
#define __LVQD__
#include "tab4vis.h"

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
/* --- vector initialization modes --- */
#define LVQ_CENTER     0        /* center of range of values */
#define LVQ_MEAN       1        /* global mean (expected value) */
#define LVQ_CLSMEAN    2        /* class  mean (expected value) */
#define LVQ_DATAPT     3        /* random training pattern */

/* --- error codes --- */
#define E_REFVEC     (-6)       /* no reference vectors */
#define E_RVINC      (-7)       /* incompatible reference vectors */

/*----------------------------------------------------------------------
  Type Definitions
----------------------------------------------------------------------*/
typedef struct {                /* --- reference vector --- */
  double x, y;                  /* coordinates */
} REFVEC;                       /* (reference vector) */

typedef struct {                /* -- l.v.q. parameters -- */
  int    imode;                 /* vector initialization mode */
  VALID  vpcls;                 /* vectors per class */
  double roff;                  /* random offset range */
  double lrate;                 /* initial learning rate */
  double decay;                 /* learning rate decay parameter */
  double delay;                 /* delay between training steps */
} LVQPAR;                       /* (l.v.q. parameters) */

/*----------------------------------------------------------------------
  Global Variables
----------------------------------------------------------------------*/
extern REFVEC     *rvecs;       /* current reference vectors and */
extern VALID      *rvcls;       /* classes they are assigned to */
extern VALID      rvcnt;        /* number of reference vectors */
extern REFVEC     **hist;       /* history of reference vectors */
extern int        histvsz;      /* size of history vector */
extern int        histcnt;      /* num. of entries in history vector */
extern LVQPAR     lvqpar;       /* l.v.q. parameters */

/*----------------------------------------------------------------------
  Functions
----------------------------------------------------------------------*/
extern void lvq_clean (void);
extern int  lvq_save  (const char *fname);
extern int  lvq_load  (const char *fname);
extern int  lvq_init  (void);
extern int  lvq_train (void);

#endif
